//////////////////////////////////////////
//                                      //
//  Bertocchi Giorgio Padovani Stefano  //
//                                      //
//                  ::.                 //
//           (\./)  .-""-.              //
//            `\'-'`      \             //
//              '.___,_^__/             //
//                                      //
//			Visual Sobriety Test		//
//////////////////////////////////////////


#include <iostream>
#include <opencv2/opencv.hpp>
#include <opencv2/core/core.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/objdetect/objdetect.hpp>

#define N_BINS 256

using namespace cv;
using namespace std;

CascadeClassifier face_cascade;
CascadeClassifier eye_cascade;



/**
* Function to detect human face and the eyes from an image.
*
* @param im The source image
* @param tpl Will be filled with the eye template, if detection success.
* @param rect Will be filled with the bounding box of the eye
* @return zero=failed, nonzero=success
*/
void detectEye(Mat& im, Mat& tpl, Rect& rect)
{
    vector<Rect> faces, eyes;
    face_cascade.detectMultiScale(im, faces, 1.1, 2, 0|CV_HAAR_SCALE_IMAGE, Size(30,30));
    for (unsigned int i = 0; i < faces.size(); i++)
    {
        Mat face = im(faces[i]);
        eye_cascade.detectMultiScale(face, eyes, 1.1, 2, 0|CV_HAAR_SCALE_IMAGE, Size(20,20));
        if (eyes.size())
        {
            rect = eyes[0] + Point(faces[i].x, faces[i].y);
            tpl = im(rect);
        }
    }
}


/**
* Perform template matching to search the user's eye in the given image.
*
* @param im The source image
* @param tpl The eye template
* @param rect The eye bounding box, will be updated with the new location of the eye
*/
void trackEye(Mat& im, Mat& tpl, Rect& rect)
{
    Size size(rect.width * 2, rect.height * 2);
    Rect window(rect + size - Point(size.width/2, size.height/2));
    window &= Rect(0, 0, im.cols, im.rows);
    Mat dst(window.width - tpl.rows + 1, window.height - tpl.cols + 1, CV_32F);

    matchTemplate(im(window), tpl, dst, CV_TM_SQDIFF_NORMED);

    double minval, maxval;
    Point minloc, maxloc;
    minMaxLoc(dst, &minval, &maxval, &minloc, &maxloc);

    if (abs(minval) <= 0.2)
    {
        rect.x = window.x + minloc.x;
        rect.y = window.y + minloc.y;

    }
    if (abs(minval) > 0.2)
    {
        rect.x = window.x + minloc.x;
        rect.y = window.y + minloc.y;

    }
    im(rect).copyTo(tpl);
    // Shows the result
    namedWindow("dst", WINDOW_AUTOSIZE);
    imshow("dst", dst);
    //
}


/**
* Perform cropping of an colorated image.
*
* @param *image The address of source image
* @param *image_out_cutted The address of output image
* @param n_col The rect width
* @param n_row The rect height
* @param orig_col The col cordinate of the rect's origin
* @param orig_row The row cordinate of the rect's origin
*/
void color_crop(Mat *image, Mat *image_out_cutted, int n_col, int n_row, int orig_col, int orig_row)
{
    if(orig_col <= image->cols-n_col && orig_row <= image->rows-n_row)
    {
        int cutted_row, cutted_col;         //starting point of cutted image
        for(cutted_row = 0; cutted_row < n_row; cutted_row++)
        {
            for(cutted_col = 0; cutted_col < n_col; cutted_col++)
            {
                image_out_cutted->at<Vec3b>(cutted_row, cutted_col)[0] = image->at<Vec3b>(orig_row+cutted_row, orig_col+cutted_col)[0];
                image_out_cutted->at<Vec3b>(cutted_row, cutted_col)[1] = image->at<Vec3b>(orig_row+cutted_row, orig_col+cutted_col)[1];
                image_out_cutted->at<Vec3b>(cutted_row, cutted_col)[2] = image->at<Vec3b>(orig_row+cutted_row, orig_col+cutted_col)[2];
            }
        }
    }
    else
    {
        cout<<"The parameters for the shift exceed the bound of original image!"<<endl;
        *image_out_cutted = *image;
    }
}


/**
* Draw all image's histogram value.
*
* @param istog[] The vector conteing the value of the histogram
* @param maxval The maximum value used for set the endscale
* @param window_name The name of the windows
*/
void plot_hist(float istog[], float maxval, string window_name)
{
    Mat window_screen = window_screen.zeros(N_BINS, 3*N_BINS, DataType<unsigned char>::type);

    float temp;

    temp = (window_screen.rows - 1)/maxval;

    for (int column=0; column<(window_screen.cols/3); column++)
    {
        for (int row = window_screen.rows-1; row>(N_BINS-floor((float)istog[column]*temp)); row--)
        {
            window_screen.at<unsigned char>(row,3*column+0) = 255;
            window_screen.at<unsigned char>(row,3*column+1) = 255;
            //window_screen.at<unsigned char>(row,3*column+2) = 0;
        }
    }
    namedWindow(window_name, WINDOW_AUTOSIZE);
    imshow(window_name, window_screen);

}


/**
* Calculate the histogram and the cumulative of a image and returns the output threshold value.
*
* @param *image The address of the image
* @param threshold_in The value of filter need in %
* @param **threshold_out1 The value of the color where start to filter
*/
void pdf(Mat *image, int threshold_in, int **threshold_out1)
{
    int temp;
    float val_area = 0;
    float maxval = 0;
    float histogram_out[N_BINS] = {};
    float cumulative_output[N_BINS] = {};

    for(int row = 0; row < image->rows; row++)
    {
        for(int column = 0; column < image->cols; column++)
        {
            temp = image->at<unsigned char>(row,column);
            histogram_out[(int)temp]+=1;
        }
    }

    for(int a = 0; a < N_BINS; a++)
    {
        val_area += histogram_out[a];

        if( histogram_out[a] > maxval)
        {
            maxval = histogram_out[a];
        }
    }

    plot_hist(histogram_out, maxval, "histogram");

    temp = 0;

    cumulative_output[0] = (histogram_out[0]/val_area);

    for(int a = 1; a < N_BINS; a++)
    {
        cumulative_output[a] = (histogram_out[a]/val_area) + cumulative_output[a-1];
    }

    for(int a = 0; a < N_BINS; a++)
    {
        if( cumulative_output[a] <= ((float)threshold_in)/100) temp = a+1;
    }

    plot_hist(cumulative_output, 1, "cumulative");

    **threshold_out1 = temp;
}


/**
* Return a new binary image of the frame filtered with the threshold value.
*
* @param *image The address of the image
* @param color_threshold The value for filtering the image
*/
Mat LPF_color_threshold(Mat *image, int color_threshold)
{
    Mat new_mat = Mat::zeros(image->rows, image->cols, image->type());
    for(int row=0; row < image->rows; row++)
    {
        for(int column=0; column < image->cols; column++)
        {

            if((image->at<unsigned char>(row,column)) < color_threshold)
                (new_mat.at<unsigned char>(row,column))=255;
            else
                (new_mat.at<unsigned char>(row,column))=0;
        }
    }
    return new_mat;
}


/**
* Return the absolute value of the distance between two points.
*
* @param x1 The number of columns of the first point
* @param y1 The number of rows of the first point
* @param x2 The number of columns of the second point
* @param y2 The number of rows of the second point
*/
float ComputeDist(float x1,float y1,float x2,float y2 )
{
    float result= sqrt( pow((x2-x1),2) + pow((y2-y1),2) );
    return result;
}


/**
* Draw in the matrix a filling circle with a given radius.
*
* @param *radius The value of the circle's radius
* @param *mask_mat The pointer at the matrix that it will be filled
*/
void CreateMask(int *radius, Mat *mask_mat)
{
    Point center = Point(*radius -1 , *radius -1);

    for(int i=0; i < mask_mat->rows; i++)
    {
        for(int j=0; j < mask_mat->cols; j++)
        {
            float distCenterPoint = ComputeDist((float)center.x, (float)center.y, (float)j, (float)i);
            if(distCenterPoint <= (float)*radius)
                mask_mat->at<unsigned char>(i,j) = 255;
            else
                mask_mat->at<unsigned char>(i,j) = 0;

        }
    }
}


/**
* Compute the match between two binary images and return the value of the space where the match is maximum.
*
* @param image The first binary image
* @param mask_mat The second binary image
* @param *center The variable where there will be save the cordinate of maximum matching
*/
void Matching(Mat image, Mat mask_mat, Point *center)
{
    Mat out = out.zeros(image.rows, image.cols, image.type());//Shows results

    Mat gravity = gravity.zeros(image.rows+mask_mat.rows, image.cols+mask_mat.cols, image.type());

    for (int row=0; row < image.rows; row++)
    {
        for (int col=0; col < image.cols; col++)
        {
            gravity.at<unsigned char>(row+ceil(mask_mat.rows/2),col+ceil(mask_mat.rows/2)) = image.at<unsigned char>(row,col);
        }
    }

    center->x = 0;
    center->y = 0;
    int maximum = 0;
    int variable = 0;

    for (int row=ceil(mask_mat.rows/2); row < (gravity.rows-ceil(mask_mat.rows/2)); row++)
    {
        for (int column=ceil(mask_mat.cols/2); column < (gravity.cols-ceil(mask_mat.cols/2)); column++)
        {
            if(gravity.at<unsigned char>(row,column) == 255)
            {
                for (int i=(row-ceil(mask_mat.rows/2)); i < (row+ceil(mask_mat.rows/2)); i++)
                {
                    for (int j=(column-ceil(mask_mat.cols/2)); j < (column+ceil(mask_mat.cols/2)); j++)
                    {
                        int a = i + mask_mat.rows/2 - row;
                        int b = j + mask_mat.cols/2 - column;
                        if( (gravity.at<unsigned char>(i,j) == 255) && (mask_mat.at<unsigned char>(a,b) == 255))
                        {
                            variable += 1;
                        }
                    }
                }
                out.at<unsigned char>(row-ceil(mask_mat.rows/2), column-ceil(mask_mat.cols/2)) = ceil(variable/4);//Shows results
                if(variable >= maximum)
                {
                    maximum = variable;
                    center->y = row - ceil(mask_mat.rows/2);
                    center->x = column - ceil(mask_mat.cols/2);
                }
                variable = 0;
            }
        }
    }

    // Shows the result
    namedWindow("Matching", WINDOW_AUTOSIZE);
    imshow("Matching", out);
    //
}


/**
* Precycle calculates the radius of the iris from the first frame, the dimension of the template and its position.
*
* @param *image The address of the image
* @param *eye_bb The address of the image
* @param *eye_tpl The address of the image
* @param threshold_in The address of the image
* @param *threshold_out The address of the image
* @param radius The value for filtering the image
* @param *element Structure to compute the dilatation
*/
bool precycle(Mat *image, Rect *eye_bb, Mat *eye_tpl, int threshold_in, int *threshold_out, int *radius, Mat *element)
{
    bool return_out = false;

    Mat gray = gray.zeros(ceil((2*eye_bb->height)/3), 2*eye_bb->width, CV_8UC1);

    cvtColor(*image, gray, CV_BGR2GRAY, 1);

    detectEye(gray, *eye_tpl, *eye_bb);

    Mat imageCutted = imageCutted.zeros(ceil((2*eye_bb->height)/3), 2*eye_bb->width, image->type());
    Mat imageCuttedBW = imageCuttedBW.zeros(ceil((2*eye_bb->height)/3), 2*eye_bb->width, CV_8UC1);

    if (eye_bb->width != 0 && eye_bb->height != 0)
    {
        color_crop(image, &imageCutted, 2*eye_bb->width, ceil((2*eye_bb->height)/3), eye_bb->x-(eye_bb->height/2), eye_bb->y+(eye_bb->width/6));

        // Shows the result
        namedWindow("Colour", WINDOW_AUTOSIZE);
        imshow("Colour", imageCutted);
        //

        cvtColor(imageCutted, imageCuttedBW, CV_BGR2GRAY, 1);

        // Shows the result
        namedWindow("B&W", WINDOW_AUTOSIZE);
        imshow("B&W", imageCuttedBW);
        //

        pdf(&imageCuttedBW, threshold_in, &threshold_out);

        cout << "precycle 1 - threshold out: " << *threshold_out << endl;

        return_out = true;
    }

    if (return_out != 0)
    {
        return_out = false;

        threshold(imageCuttedBW, imageCuttedBW, *threshold_out, 255, cv::THRESH_BINARY_INV);

        // Shows the result
        namedWindow("B&W cutted image", WINDOW_AUTOSIZE);
        imshow("B&W cutted image", imageCuttedBW);
        //

        dilate(imageCuttedBW, imageCuttedBW, *element);

        // Shows the result
        namedWindow("B&W dilatate image", WINDOW_AUTOSIZE);
        imshow("B&W dilatate image", imageCuttedBW);
        //

        Mat sumCol = sumCol.zeros(imageCuttedBW.size(), imageCuttedBW.type());
        Mat sumRow = sumRow.zeros(imageCuttedBW.size(), imageCuttedBW.type());
        Mat sumXY = sumXY.zeros((int)imageCuttedBW.rows, 1, CV_8UC1);//vector
        Mat sumYX = sumYX.zeros(1, (int)imageCuttedBW.cols, CV_8UC1);//vector

        int ndifferenttozeroR = 0;
        int nsumR = 0;
        int ndifferenttozeroC = 0;
        int nsumC = 0;

        for (int a=0; a<imageCuttedBW.rows; a++)
        {
            int sommax = 0;
            for (int b=0; b<imageCuttedBW.cols; b++)
            {
                if(imageCuttedBW.at<unsigned char>(a,b)>=*threshold_out) sommax += 1;
            }

            sumXY.at<unsigned char>(a,0) = sommax;

            if(sommax != 0)
            {
                ndifferenttozeroR++;
                nsumR += sommax;
            }

            for (int b=0; b<sommax; b++)
            {
                sumRow.at<unsigned char>(a,b) = 255;
            }
        }

        for (int a=0; a<imageCuttedBW.cols; a++)
        {
            int sommay = 0;
            for (int b=0; b<imageCuttedBW.rows; b++)
            {
                if(imageCuttedBW.at<unsigned char>(b,a)>=*threshold_out) sommay += 1;
            }

            sumYX.at<unsigned char>(0,a) = sommay;

            if(sommay != 0)
            {
                ndifferenttozeroC++;
                nsumC += sommay;
            }

            for (int b=0; b<sommay; b++)
            {
                sumCol.at<unsigned char>(b,a) = 255;
            }

        }

        float meanvalueC = 0.0;
        meanvalueC = nsumC/ndifferenttozeroC;
        int height_int = (int)sumXY.rows;
        int width_int = (int)sumYX.cols;

        height_int = ndifferenttozeroR;

        for (int a=0; a<sumYX.cols; a++)
        {
            if(sumYX.at<unsigned char>(0,a) <= (int)meanvalueC)
            {
                imageCuttedBW.col(a) = imageCuttedBW.col(a).zeros((int)imageCuttedBW.rows, 1, CV_8UC1);
                width_int--;
            }
        }

        *radius = ceil ( 0.25 * (width_int + height_int) );

        cout << "precycle 2 - height: " << height_int << " width: " << width_int << " radius: " << *radius << '\n';

        // Shows the results
        namedWindow("B&W cutted filtred image1", WINDOW_AUTOSIZE);
        imshow("B&W cutted filtred image1", imageCuttedBW);

        namedWindow("B&W sum on the columns", WINDOW_AUTOSIZE);
        imshow("B&W sum on the columns", sumCol);

        namedWindow("B&W sum on the rows", WINDOW_AUTOSIZE);
        imshow("B&W sum on the rows", sumRow);
        //

        return_out = true;
    }

    return return_out;
}


/**
* Plot in auto-adattive and auto-updated modes the shape of the input vector.
*
* @param distance[] The vector conteing the values that will be plot
* @param n_frame The number of the current frame in the video, necessary variable for update the plot
* @param window_name The name of the windows
*/
void display_distance(float distance[], int n_frame, string window_name)
{
    int max = distance[0];
    int min = distance[0];
    int lenght_D = 100;
    int height_D = 200;

    for (int index = 0; index < n_frame; index++)
    {
        if(max <= distance[index]) max=distance[index];
        if(min >= distance[index]) min=distance[index];
    }

    if ((n_frame+lenght_D/5) >= lenght_D) lenght_D = n_frame + lenght_D/5;
    if ((ceil(max-min)*1.5) >= height_D) height_D = ceil(ceil(max-min)*1.5);

    Mat window_screen = window_screen.zeros(height_D, 3*lenght_D, DataType<unsigned char>::type);

    for (int column=0; column<n_frame; column++)
    {
        float y = ( ((distance[column] - min)/(max-min)) * 4 + 1) * (height_D/6);
        int x = ceil( y );

        window_screen.at<unsigned char>(x,3*column+0) = 255;
        window_screen.at<unsigned char>(x,3*column+1) = 255;
        //window_screen.at<unsigned char>(row,3*column+2) = 0;
    }

    namedWindow(window_name, WINDOW_AUTOSIZE);
    imshow(window_name, window_screen);

}


/**
* Rotate the image by a given angle.
*
* @param src The address of the source image
* @param angle The amount of the rotation
* @param dst The address of the destination image
*/
void rotate(Mat& src, float angle, Mat& dst)
{
    int len = max(src.cols, src.rows);
    Point2f pt(len/2., len/2.);
    Mat r = getRotationMatrix2D(pt, angle, 1.0);
    warpAffine(src, dst, r, Size(len, len));
}


/**
* The main.
*/
int main()
{
    // Variabiles
    bool val_okey = false;
    int n_frame = 0;
    int threshold_in = 96;// xx%, experimental value.
    int threshold_out = 0;
    int radius = 0;
    int dilation = 3;// x, experimental value.
    char name[] = "XXXXX.mov";// "x", location & name of the input video.

    // Load the model for the cascade
    face_cascade.load("haarcascade_frontalface_default.xml");
    eye_cascade.load("haarcascade_eye_tree_eyeglasses.xml");

    // Open webcam '0' video 'URL'
    VideoCapture cap;
    cap.open(name);

    // Check if everything is ok
    if (face_cascade.empty() || eye_cascade.empty() || !cap.isOpened())
        return 1;

    // Set video to 1280x720
    cap.set(CV_CAP_PROP_FRAME_WIDTH, 1280);
    cap.set(CV_CAP_PROP_FRAME_HEIGHT, 720);

    // Mat and vect
    Mat frame, primo_frame, gray, eye_tpl;
    Rect eye_bb;
    Mat imageCutted, imageCuttedBW, imageCuttedFiltred, iris_mask;
    Point center;
    float vector_dist[1500];
    float vector_dist_filtered[1500];
    Mat element = getStructuringElement(MORPH_ELLIPSE, Size(2*dilation+1, 2*dilation+1), Point(dilation, dilation));

    // Elaboration pre cycle
    while (!val_okey)
    {
        cap >> primo_frame;

        flip(primo_frame, primo_frame, 1);
        //rotate(primo_frame, 180, primo_frame);//in the case of a rotated video

        val_okey = precycle(&primo_frame, &eye_bb, &eye_tpl, 100-threshold_in, &threshold_out, &radius, &element);

        // Shows the result
        rectangle(primo_frame, eye_bb, CV_RGB(0,255,0));
        namedWindow("Primo frame", WINDOW_AUTOSIZE);
        imshow("Primo frame", primo_frame);
        //
    }

    // Re-upload webcam '0' or video 'URL'
    VideoCapture video;
    video.open(name);

    // Create the iris mask
    iris_mask = iris_mask.zeros(2*radius, 2*radius, frame.type());
    CreateMask(&radius, &iris_mask);

    // Shows the result
    namedWindow("iris_mask", WINDOW_AUTOSIZE);
    imshow("iris_mask", iris_mask);
    //

    cout << "precycle: V" << endl;

    cvWaitKey(0);
    destroyAllWindows();

    // Real cycle
    while (waitKey(15) != 'q')
    {
        video >> frame;
        if (frame.empty())
            break;

        // Flip the frame horizontally and rotate it
        flip(frame, frame, 1);
        //rotate(frame, 180, frame);//in the case of a rotated video

        // Convert to grayscale and adjust the image contrast using histogram equalization
        cvtColor(frame, gray, CV_BGR2GRAY);

        // Track the moviment of the eye
        trackEye(gray, eye_tpl, eye_bb);

        // Crop
        imageCutted = imageCutted.zeros(ceil((2*eye_bb.height)/3), 2*eye_bb.width, frame.type());
        color_crop(&frame, &imageCutted, 2*eye_bb.width, ceil((2*eye_bb.height)/3), eye_bb.x-(eye_bb.height/2), eye_bb.y+(eye_bb.width/6));

        // Greyscaling
        cvtColor(imageCutted, imageCuttedBW, CV_BGR2GRAY);

        imageCuttedFiltred = imageCuttedFiltred.zeros(2*eye_bb.height, 2*eye_bb.width, gray.type());

        // Filtering
        imageCuttedFiltred = LPF_color_threshold(&imageCuttedBW, threshold_out);

        // Morphological operation
        dilate(imageCuttedFiltred, imageCuttedFiltred, element);

        // Matching
        Matching(imageCuttedFiltred, iris_mask, &center);

        // Calculate distance
        vector_dist[n_frame] = ComputeDist(0.0, (float)(eye_bb.height/2), (float)center.x, (float)center.y);
        vector_dist_filtered[n_frame] = vector_dist[n_frame];

        // Plots results
        display_distance(vector_dist, n_frame, "Distance");

        // Shows result
        circle(imageCutted, center, 3, Scalar(0,0,255), -1, 8, 0 );
        circle(imageCutted, center, radius, Scalar(255,255,0), 1, 8, 0 );
        namedWindow("Cutted", WINDOW_AUTOSIZE);
        imshow("Cutted", imageCutted);

        namedWindow("filtred", WINDOW_AUTOSIZE);
        imshow("filtred", imageCuttedFiltred);
        //

        // Count number of frames
        n_frame++;

    }
    cout << "real cycle 1 - n frame: " << n_frame << endl;
    cout << "real cycle: V" << endl;

    cvWaitKey(0);
    destroyAllWindows();

    // Post Processing
    for(int index = 2; index<n_frame-2; index++)
    {
        vector_dist_filtered[index]=(vector_dist[index-2]+vector_dist[index-1]+vector_dist[index]+vector_dist[index+1]+vector_dist[index+2])/5;
    }

    // Plots results
    display_distance(vector_dist_filtered, n_frame, "Distance_filtred");

    // Convolution
    float step = 16;// x, experimental value that must be always even.
    float min = vector_dist_filtered[0];
    float max = vector_dist_filtered[0];
    float max_T, min_T;

    float tri[(int)step+1];
    float fake_curva[n_frame+(int)step];

    // Curve for the convolution
    for(int index = 0; index<(n_frame+step); index++)
    {
        if(index<(step/2))
        {
            fake_curva[index] = vector_dist_filtered[0];
        }
        else if(index>=(n_frame+step/2))
        {
            fake_curva[index] = vector_dist_filtered[n_frame-1];
        }
        else
        {
            fake_curva[index] = vector_dist_filtered[index-(int)(step/2)];
        }

        if(min > fake_curva[index])
        {
            min=fake_curva[index];
        }
        if(max < fake_curva[index])
        {
            max=fake_curva[index];
        }
    }

    // Tri
    for(int index = 0; index<=step; index++)
    {
        float x = ((25*(max-min))/90) * (2/step) * ((step/2) - fabs((step/2)-index));
        tri[index] = x;
    }

    max_T = min_T = tri[0];

    for(int j = 0; j<step+1; j++)
    {
        if(min_T > tri[j]) min_T=tri[j];
        if(max_T < tri[j]) max_T=tri[j];
    }

    // Matrices for plot results
    Mat plot_fake_curva = plot_fake_curva.zeros(100+2, n_frame+(int)step, CV_32FC1);
    Mat plot_tri1 = plot_tri1.zeros(ceil(100*(max_T-min_T)/(max-min))+2*dilation+2, step+1, CV_32FC1);
    Mat plot_tri2 = plot_tri2.zeros(ceil(100*(max_T-min_T)/(max-min))+2*dilation+2, step+1, CV_32FC1);

    for(int j = 0; j<n_frame+(int)step; j++)
    {
        plot_fake_curva.at<float>(ceil((100*(fake_curva[j]-min))/(max-min))+1, j) = 1;
    }

    for(int j = 0; j<step+1; j++)
    {
        plot_tri1.at<float>((int)ceil((100*tri[j])/(max-min))+dilation+1, j) = 1;
        plot_tri2.at<float>(plot_tri2.rows-((int)ceil((100*tri[j])/(max-min))+dilation+1), j) = 1;
    }

    // Introducing a tollerance for the tri
    Mat element1 = getStructuringElement(MORPH_ELLIPSE, Size(dilation+1, dilation+1), Point(dilation, dilation));
    dilate(plot_tri1, plot_tri1, element1);
    dilate(plot_tri2, plot_tri2, element1);

    // Shows result
    namedWindow("Tri up", WINDOW_AUTOSIZE);
    imshow("Tri up", 255*plot_tri1);
    namedWindow("Tri down", WINDOW_AUTOSIZE);
    imshow("Tri down", 255*plot_tri2);

    // Search the maximum
    Mat dst1(plot_fake_curva.rows - plot_tri1.rows + 1, plot_fake_curva.cols - plot_tri1.cols + 1, CV_32F);
    matchTemplate(plot_fake_curva, plot_tri1, dst1, CV_TM_CCORR);
    Mat dst2(plot_fake_curva.rows - plot_tri2.rows + 1, plot_fake_curva.cols - plot_tri2.cols + 1, CV_32F);
    matchTemplate(plot_fake_curva, plot_tri2, dst2, CV_TM_CCORR);

    min = max = dst1.at<float>(0,0);

    for(int j = 0; j<dst1.rows; j++)
    {
        for(int k = 0; k<dst1.cols; k++)
        {
            if(dst1.at<float>(j, k) < min)
            {
                min = dst1.at<float>(j, k);
            }
            if(dst2.at<float>(j, k) < min)
            {
                min= dst2.at<float>(j, k);
            }
            if(dst1.at<float>(j, k) > max)
            {
                max= dst1.at<float>(j, k);
            }
            if(dst2.at<float>(j, k) > max)
            {
                max= dst2.at<float>(j, k);
            }
        }
    }

    // Cumulative & array for the jerky
    float a, b;
    float final[dst1.cols];

    for(int j = 0; j<dst1.cols; j++)
    {

        a = b = 0;

        for(int k = 0; k<dst1.rows; k++)
        {
            dst1.at<float>(k, j) = 100*(dst1.at<float>(k, j)-min)/(max-min);
            dst2.at<float>(k, j) = 100*(dst2.at<float>(k, j)-min)/(max-min);
            if(dst1.at<float>(k, j) > 75)// 75%
            {
                a = a + dst1.at<float>(k,j);
            }
            if(dst2.at<float>(k, j) > 75)// 75%
            {
                b = b + dst2.at<float>(k,j);
            }
        }

        if(a > b)
        {
            final[j] = a/dst1.rows;
        }
        else
        {
            final[j] = b/dst1.rows;
        }
    }

    // Plots rough results
    display_distance(final, dst1.cols-1, "Crosscorrelation");

    max = final[0];

    for(int j = 0; j<dst1.cols; j++)
    {
        if(max<final[j]) max=final[j];
    }

    // Softening plot
    for(int j = 0; j<dst1.cols; j++)
    {
        if(final[j]>(0.25*max)) final[j]=0;
        else final[j]=100;
    }
    for(int j = 0; j<dst1.cols; j++)
    {
        if(final[j]==0)
        {
            final[j-2]=0;
            final[j-1]=0;
        }
    }
    for(int j = dst1.cols-1; j>=0; j--)
    {
        if(final[j]==0)
        {
            final[j+1]=0;
            final[j+2]=0;
        }
    }

    // Plots results
    display_distance(final, dst1.cols-1, "Drunk jerky");

    cout << "post cycle: V" << endl;

    cvWaitKey(0);
    cvWaitKey(0);
    destroyAllWindows();

    return 0;
}
